
// This file was autogenerated by go-to-protobuf. Do not edit it manually!

syntax = 'proto2';

package github.com.openshift.origin.pkg.project.api.v1;

import "k8s.io/kubernetes/pkg/api/unversioned/generated.proto";
import "k8s.io/kubernetes/pkg/api/v1/generated.proto";
import "k8s.io/kubernetes/pkg/runtime/generated.proto";
import "k8s.io/kubernetes/pkg/util/intstr/generated.proto";

// Package-wide variables from generator "generated".
option go_package = "v1";

// Projects are the unit of isolation and collaboration in OpenShift. A project has one or more members,
// a quota on the resources that the project may consume, and the security controls on the resources in
// the project. Within a project, members may have different roles - project administrators can set
// membership, editors can create and manage the resources, and viewers can see but not access running
// containers. In a normal cluster project administrators are not able to alter their quotas - that is
// restricted to cluster administrators.
// 
// Listing or watching projects will return only projects the user has the reader role on.
// 
// An OpenShift project is an alternative representation of a Kubernetes namespace. Projects are exposed
// as editable to end users while namespaces are not. Direct creation of a project is typically restricted
// to administrators, while end users should use the requestproject resource.
message Project {
  // Standard object's metadata.
  optional k8s.io.kubernetes.pkg.api.v1.ObjectMeta metadata = 1;

  // Spec defines the behavior of the Namespace.
  optional ProjectSpec spec = 2;

  // Status describes the current status of a Namespace
  optional ProjectStatus status = 3;
}

// ProjectList is a list of Project objects.
message ProjectList {
  // Standard object's metadata.
  optional k8s.io.kubernetes.pkg.api.unversioned.ListMeta metadata = 1;

  // Items is the list of projects
  repeated Project items = 2;
}

// ProjecRequest is the set of options necessary to fully qualify a project request
message ProjectRequest {
  // Standard object's metadata.
  optional k8s.io.kubernetes.pkg.api.v1.ObjectMeta metadata = 1;

  // DisplayName is the display name to apply to a project
  optional string displayName = 2;

  // Description is the description to apply to a project
  optional string description = 3;
}

// ProjectSpec describes the attributes on a Project
message ProjectSpec {
  // Finalizers is an opaque list of values that must be empty to permanently remove object from storage
  repeated string finalizers = 1;
}

// ProjectStatus is information about the current status of a Project
message ProjectStatus {
  // Phase is the current lifecycle phase of the project
  optional string phase = 1;
}

