// This is the source repository for OpenShift Origin - the best way to build, manage, and deploy
// applications in the cloud.  The OpenShift 3.0 codebase is based around Docker images and containers
// and the Kubernetes container management system.
package origin
