// This package is generated by client-gen with arguments: --clientset-api-path=/oapi --clientset-name=release_v1_5 --clientset-path=github.com/openshift/origin/pkg/project/client/clientset_generated --go-header-file=hack/boilerplate.txt --input=[api/v1] --input-base=github.com/openshift/origin/pkg/project/api --output-base=../../..

// This package has the automatically generated clientset.
package release_v1_5
