// Package latest defines the default output serializations that code should
// use and imports the required schemas.  It also ensures all previously known
// and supported API versions are available for conversion. Consumers may
// import this package in lieu of importing individual versions.
package latest
