// Package extensions defines cryptographic extensions for OpenShift. This
// package contains x509 extension object identifier constants and helpers
// for generating certificates on an OpenShift cluster.
package extensions
