// Package api includes all OpenShift-specific types used to communicate
// between the various parts of the OpenShift and the Kubernetes systems.
//
// Unlike the upstream Kubernetes, API objects in OpenShift are separated
// into individual packages.
package api
