// This package is generated by client-gen with arguments: --clientset-api-path=/oapi --clientset-name=internalclientset --clientset-path=github.com/openshift/origin/pkg/build/client/clientset_generated --go-header-file=hack/boilerplate.txt --input=[api/] --input-base=github.com/openshift/origin/pkg/build/api --output-base=../../..

// This package has the automatically generated clientset.
package internalclientset
