package v1

// This file contains methods that can be used by the go-restful package to generate Swagger
// documentation for the object types found in 'types.go' This file is automatically generated
// by hack/update-generated-swagger-descriptions.sh and should be run after a full build of OpenShift.
// ==== DO NOT EDIT THIS FILE MANUALLY ====

var map_ProjectLimitBySelector = map[string]string{
	"":            "ProjectLimitBySelector specifies the maximum number of projects allowed for a given user label selector",
	"selector":    "Selector is a user label selector. An empty selector selects everything.",
	"maxProjects": "MaxProjects is the number of projects allowed for this class of users. If MaxProjects is nil, there is no limit to the number of projects users can request. An unlimited number of projects is useful in the case a limit is specified as the default for all users and only users with a specific set of labels should be allowed unlimited project creation.",
}

func (ProjectLimitBySelector) SwaggerDoc() map[string]string {
	return map_ProjectLimitBySelector
}

var map_ProjectRequestLimitConfig = map[string]string{
	"":                              "ProjectRequestLimitConfig is the configuration for the project request limit plug-in It contains an ordered list of limits based on user label selectors. Selectors will be checked in order and the first one that applies will be used as the limit.",
	"limits":                        "Limits are the project request limits",
	"maxProjectsForSystemUsers":     "MaxProjectsForSystemUsers controls how many projects a certificate user may have.  Certificate users do not have any labels associated with them for more fine grained control",
	"maxProjectsForServiceAccounts": "MaxProjectsForServiceAccounts controls how many projects a service account may have.  Service accounts can't create projects by default, but if they are allowed to create projects, you cannot trust any labels placed on them since project editors can manipulate those labels",
}

func (ProjectRequestLimitConfig) SwaggerDoc() map[string]string {
	return map_ProjectRequestLimitConfig
}
